/*
 *  SThan1 GPIO Support
 *
 *  Copyright (c) 2008 Richard Liu,  STMicroelectronics  <richard.liu@st.com>
 *  Copyright (c) 2008-2010 Arnaud Patard <apatard@mandriva.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 */

#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/module.h>
#include <linux/spinlock.h>
#include <linux/err.h>
#include <asm/types.h>
#include <asm/io.h>
#include <linux/gpio.h>

//0xd0018100  gpio data out
//0xd0018140  gpio data out
//0xd0018104  gpio data out en
//0xd0018144  gpio data out en
//0xd0018108  brink
//0xd0018148  brink
//0xd0018110  gpio data in
//0xd0018150  gpio data in




#include "linux/init.h"
#include "linux/slab.h"
#include <linux/platform_device.h>
 
#define GPIO_PANEL_UP    13
#define GPIO_PANEL_DOWN  14
#define GPIO_PANEL_RIGHT 15
#define GPIO_PANEL_LEFT  16
#define GPIO_PANEL_POWER 12
 
#define KEYS_POLL_INTERVAL         20      /* msecs */
#define KEYS_DEBOUNCE_INTERVAL     (3 * KEYS_POLL_INTERVAL)
 
 
static struct gpio_keys_button _gpio_keys[] __initdata = {
{
        .desc           = "sw1",
        .type           = EV_KEY,
        .code           = BTN_0,
        .debounce_interval = KEYS_DEBOUNCE_INTERVAL,
        .gpio           = AP81_GPIO_BTN_SW1,
        .active_low     = 1,
} , {
        .desc           = "sw4",
        .type           = EV_KEY,
        .code           = BTN_1,
        .debounce_interval = KEYS_DEBOUNCE_INTERVAL,
        .gpio           = AP81_GPIO_BTN_SW4,
        .active_low     = 1,
}
};


void __init ath79_register_gpio_keys_polled(int id,
                                            unsigned poll_interval,
                                            unsigned nbuttons,
                                            struct gpio_keys_button *buttons)
{
        struct platform_device *pdev;
        struct gpio_keys_platform_data pdata;
        struct gpio_keys_button *p;
        int err;

        p = kmalloc(nbuttons * sizeof(*p), GFP_KERNEL);
        if (!p)
                return;

        memcpy(p, buttons, nbuttons * sizeof(*p));

        pdev = platform_device_alloc("gpio-keys-polled", id);
        if (!pdev)
                goto err_free_buttons;

        memset(&pdata, 0, sizeof(pdata));
        pdata.poll_interval = poll_interval;
        pdata.nbuttons = nbuttons;
        pdata.buttons = p;

        err = platform_device_add_data(pdev, &pdata, sizeof(pdata));
        if (err)
                goto err_put_pdev;

        err = platform_device_add(pdev);
        if (err)
                goto err_put_pdev;

        return;

err_put_pdev:
        platform_device_put(pdev);

err_free_buttons:
        kfree(p);
}


static void __init ap81_setup(void)
{
        u8 *cal_data = (u8 *) KSEG1ADDR(AP81_CAL_DATA_ADDR);

        ath79_register_leds_gpio(-1, ARRAY_SIZE(ap81_leds_gpio),
                                 ap81_leds_gpio);
        ath79_register_gpio_keys_polled(-1, AP81_KEYS_POLL_INTERVAL,
                                        ARRAY_SIZE(ap81_gpio_keys),
                                        ap81_gpio_keys);
        ath79_register_spi(&ap81_spi_data, ap81_spi_info,
                           ARRAY_SIZE(ap81_spi_info));
        ath79_register_ar913x_wmac(cal_data);
}













































#define GPIO_BASE 0xd0018110
#define GPIO_SHIFT 1
#define GPIO_ARIA_SIZE 2


static DEFINE_SPINLOCK(gpio_lock);

int gpio_get_value(unsigned gpio)
{
	struct resource *addr;
	spin_lock(&gpio_lock);
	addr = ioremap_nocache(GPIO_BASE+GPIO_SHIFT, GPIO_ARIA_SIZE);
	//printk( KERN_ERR "read_pin : addr %u\n",addr);
	u16 value     = ioread16(addr);
	iounmap(addr);
	spin_unlock(&gpio_lock);
	const int shift = gpio - (GPIO_SHIFT*8);
	//printk( KERN_ERR "read_pin : shift %d v %d\n",shift,((value << shift) & 1));
	return ((value >> shift) & 1) == 0;
}
EXPORT_SYMBOL(gpio_get_value);

void gpio_set_value(unsigned gpio, int state)
{
	struct resource *addr;
	u32 mask,value;
	spin_lock(&gpio_lock);
	if(gpio<32){
		addr = ioremap_nocache(0xd0018100, 4);
		mask = 1 << gpio;
		value = ioread32(addr);
		if(state){
			value = value | mask;
		}else{
			value = value & (~mask);
		}
		iowrite32(value, addr);
		iounmap(addr);
	}else if(gpio<64){
		addr = ioremap_nocache(0xd0018140, 4);
		mask = 1 < (gpio-32);
		value = ioread32(addr);
		if(state){
			value = value | mask;
		}else{
			value = value & (~mask);
		}
		iowrite32(value, addr);
		iounmap(addr);
	}
	spin_unlock(&gpio_lock);
}
EXPORT_SYMBOL(gpio_set_value);

static int han1_gpio_direction_input(struct gpio_chip *chip, unsigned gpio){
	return 0;
}

static int han1_gpio_direction_output(struct gpio_chip *chip,
		unsigned gpio, int level){
	return 0;
}

static int han1_gpio_get_value(struct gpio_chip *chip, unsigned gpio)
{
	return gpio_get_value(gpio);
}

static void han1_gpio_set_value(struct gpio_chip *chip,
		unsigned gpio, int value)
{
	gpio_set_value(gpio, value);
}

static struct gpio_chip han1_chip = {
	.label                  = "han1",
	.direction_input        = han1_gpio_direction_input,
	.get                    = han1_gpio_get_value,
	.direction_output       = han1_gpio_direction_output,
	.set                    = han1_gpio_set_value,
	.base                   = 0,
	.ngpio                  = 0,
};

static int __init han1_gpio_setup(void)
{
	return gpiochip_add(&han1_chip);
}
arch_initcall(han1_gpio_setup);
