/*
 *  Driver routines for BUFFALO Platform
 *
 *  Copyright (C)  BUFFALO INC.
 *
 *  This software may be used and distributed according to the terms of
 *  the GNU General Public License (GPL), incorporated herein by reference.
 *  Drivers based on or derived from this code fall under the GPL and must
 *  retain the authorship, copyright and license notice.  This file is not
 *  a complete program and may only be used when the entire operating
 *  system is licensed under the GPL.
 *
 */

#include <linux/module.h>
#include <linux/proc_fs.h>
#include <linux/init.h>
#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm-generic/errno-base.h>
#include <linux/cdev.h>
#include <linux/errno.h>
#include <linux/fs.h>
#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/module.h>
#include <linux/random.h>
#include <linux/unistd.h>
#include <linux/vmalloc.h>
#include <linux/version.h>
#include <linux/delay.h>
#include <linux/crypto.h>
#include <linux/scatterlist.h>
#include <linux/proc_fs.h>
#include <linux/spi/spi.h>
#include <linux/spi/spidev.h>
#include <linux/spi/spi_gpio.h>
#include <asm/segment.h>

#include "mvCommon.h"
#include "mvOs.h"
#include "spi/mvSpi.h"
#include "boardEnv/mvBoardEnvSpec.h"
#include "boardEnv/mvBoardEnvLib.h"


/* Definitions */
#define BUFFALO_DRIVER_VER	"1.00"
#define DEV_NAME		"buf_spi"
#define AUTHOR			"(C) BUFFALO INC."
#define DEV_NUM 1
#define DEV_MINOR_NO 1

#define DEBUG

//#if defined(CONFIG_ARCH_ARMADA370)

struct spi_dev {
	struct class *class;    /** driver class */
	struct cdev cdev;
	dev_t dev_id;
};

static struct spi_dev m_dev;

//fopen
static int _open(struct inode* inode, struct file* filp) {
	struct spi_dev *dev = container_of(inode->i_cdev, struct spi_dev, cdev);
#ifdef DEBUG
	printk( KERN_INFO "spi1 : open\n");
#endif
	filp->private_data = dev;
	return 0;
}

//fclose
static int _release(struct inode* inode, struct file* filp) {
#ifdef DEBUG
	printk( KERN_INFO "spi1 : release\n");
#endif
	return 0;
}


//fwrite
static ssize_t _write(struct file* filp, const char* buf, size_t count, loff_t* pos) {
#ifdef DEBUG
	printk( KERN_INFO "spi1 : write\n" );
#endif
	//struct spi_dev *dev = filp->private_data;

	MV_U8 spiId = 1;
	MV_U8 *pTxBuff = buf;
	MV_U32 buffSize = count;

	mm_segment_t oldfs = get_fs();
	set_fs(get_ds());
	mvSpiCsAssert(spiId);
	MV_STATUS status = mvSpiWrite(spiId, pTxBuff, buffSize);
	mvSpiCsDeassert(spiId);
	printk( KERN_INFO "fb_server :vfs_write %d\n",status);
	set_fs(oldfs);

	return count;
}


////////////////////////////////////////
//module
static struct file_operations m_fops = {
		owner   : THIS_MODULE,
		write   : _write,
		open    : _open,
		release : _release,
};

void BuffaloSpi1_exit(void) {

	if(m_dev.class){
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,27))
#else
		//devファイル削除
		int num,devno;
		const int majorNo = MAJOR(m_dev.dev_id);
		for ( num = 0; num < DEV_NUM; num++ ) {
			devno = MKDEV(majorNo, DEV_MINOR_NO + num);
			device_destroy( m_dev.class, devno );
		}
#endif
		class_destroy( m_dev.class );
	}

	if(&m_dev.cdev){
		cdev_del(&m_dev.cdev);
	}

	if(m_dev.dev_id){
		unregister_chrdev_region(m_dev.dev_id, DEV_NUM);
	}

#ifdef DEBUG
	printk( KERN_INFO "spi1 : cleanup\n" );
#endif
}

int __init BuffaloSpi1_init(void) {

#ifdef DEBUG
	printk( KERN_INFO "#################spi1 : init\n" );
#endif

	int result = -EINVAL;

	if (alloc_chrdev_region(&m_dev.dev_id, DEV_MINOR_NO, DEV_NUM, DEV_NAME)) {
		printk( KERN_ERR "spi1 : alloc_chrdev_region error\n" );
		result = -EBUSY;
		goto err;
	}

	cdev_init(&m_dev.cdev, &m_fops);
	m_dev.cdev.owner = THIS_MODULE;
	m_dev.cdev.ops   = &m_fops;
	if (cdev_add(&m_dev.cdev, m_dev.dev_id, DEV_NUM)) {
		printk( KERN_ERR "spi1 : cdev_add error\n" );
		result = -EBUSY;
		goto err;
	}


	//dev内に登録
	int num,devno;
	int majorNo = MAJOR(m_dev.dev_id);
	m_dev.class = class_create( THIS_MODULE, DEV_NAME );
	if(!m_dev.class){
		result = -EBUSY;
		goto err;
	}
	for ( num = 0; num < DEV_NUM; num++ ) {
		devno = MKDEV(majorNo, DEV_MINOR_NO + num);
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,27))
		struct device *dev = class_device_create( m_dev.class , NULL, devno, NULL, DEV_NAME"%d", num );
#else
		struct device *dev = device_create( m_dev.class , NULL, devno, NULL, DEV_NAME"%d", num );
#endif
		if (!dev) {
			printk( KERN_ERR "spi1 : cannot create device error\n" );
			result = -EBUSY;
			goto err;
		}
	}

	result = 0;
	printk("%s %s Ver.%s installed.\n", DEV_NAME, AUTHOR, BUFFALO_DRIVER_VER);
	return result;

err:
	//エラー処理系
	BuffaloSpi1_exit();//ロールバック
	return result;
}

module_init(BuffaloSpi1_init);
module_exit(BuffaloSpi1_exit);

//#endif

/* Module parameters */
MODULE_AUTHOR(AUTHOR);
MODULE_DESCRIPTION(DEV_NAME);
MODULE_LICENSE("GPL");
MODULE_VERSION(BUFFALO_DRIVER_VER);

