/********************************************************************************/
/***                OLED Display Dot All ON                                   ***/
/***                                                                          ***/
/***                Name              : LD7030_MX728_Sample_v100.c            ***/
/***                Create            : Tohoku Pioneer Corporation            ***/
/***                Copyright(c) 2010 : Tohoku Pioneer Corporation            ***/
/***                Date              : 2012.9.28                             ***/
/***                Version           : V1.00                                 ***/
/********************************************************************************/
/* Handling precautions for this file

  This file and the copyright of this file shall remain the property of 
  Tohoku Pioneer Corporation.

  This file is provided as reference data which is used in the course of 
  designing your products which include Tohoku Pioneer Corporation's OLED products.

  This file shall be used only for the purpose of designing your products 
  which include Tohoku Pioneer Corporation's OLED products.

  The source code in this file is made on the assumption that it would be used 
  under certain circumstances for some kinds of hardware.  Tohoku Pioneer Corporation
  does not warrant that the source code operates without any modification 
  under your company's circumstance.

  It shall be prohibited to use this file secondarily 
  (i.e. reprint, cites, rewrites or edits).

  Tohoku Pioneer Corporation shall not be liable for any direct or indirect damages,
  losses, or costs arising from any usage of the source code in this file.

 */

#include "fb_server.h"

#ifndef DUMMY_DISP

#include <linux/semaphore.h>
#include <asm/segment.h>
#include <linux/kthread.h>

#ifdef CONFIG_BUFFALO_PLATFORM
#include "spi/mvSpi.h"
#endif

static unsigned char m_spiId = 1;

/////////////////////////////////////////////
//セマフォ
static struct semaphore g_sem;
static void tz_init_lock(void){
	sema_init(&g_sem, 1);
}
static void lock(void){
	down(&g_sem);
}
static bool trylock(void){
	return down_trylock(&g_sem)==0;
}
static void unlock(void){
	up(&g_sem);
}

/******************************/
/*** type definition          */
/******************************/
typedef unsigned char   BYTE;
typedef unsigned short  WORD;

/******************************/
/*** Panel Constant         ***/
/******************************/
#define EL_WIDTH        256                           /* Display width Dot Size       */
#define EL_HEIGHT       64                            /* Display height Size          */
#define EL_BYTE_SIZE    (EL_WIDTH * EL_HEIGHT / 8)    /* Screen Display Byte Size     */



/**************************************/
/**** Command Definition          *****/
/**************************************/
typedef struct tagM {
	BYTE  cmdsize;    /* Include Cmd Byte */
	BYTE  cmd;        /* Command Address  */
	BYTE  dat1;       /* Command Data1    */
	BYTE  dat2;       /* Command Data2    */
	BYTE  dat3;       /* Command Data3    */
	BYTE  dat4;       /* Command Data4    */
} M;

typedef struct tag_cmdreg_LD7030 {
	M   SOFTRES;		/* Software Reset */
	M   DDISPONOFF;		/* Dot Matrix Display ON/OFF */
	M   DispDirection;	/* Display Direction */
	M   Reserved1;
	M   Reserved2;
	M   Reserved3;
	M   DSTBYONOFF;		/* Dot Matrix Display Stand-by ON/OFF */
	M   Reserved4;
	M   Reserved5;
	M   Reserved6;
	M   Reserved7;
	M   Reserved8;
	M   WriteDirection;	/* Graphics Memory Writing Direction */
	M   DispSizeX;		/* Display Size X */
	M   DispSizeY;		/* Display Size Y */
	M   XBoxAdrsSTART;	/* Data Reading/Writing Box */
	M   XBoxAdrsEND;
	M   YBoxAdrsSTART;
	M   YBoxAdrsEND;
	M   XDispStart;		/* Display Start X */
	M   YDispStart;		/* Display Start Y */
	M   Reserved9;
	M   SCLK;			/* System Clock division-ratio Setting */
	M   DDEN;			/* Setting the STBY Pin */
	M   Reserved10;
	M   FDIM;			/* Dimmer Setting */
	M   Reserved11;
	M   DispMode;		/* Display Mode Setting */
	M   Reserved12;
	M   Reserved13;
	M   CAreaSizeY;		/* CArea Size Y Setting */
	M   ColorChange1;	/* Color Change Setting */
	M   ColorChange2;
	M   ColorChange3;
	M   Reserved14;
	M   CmdEND;			/* Command END  */
	M   DataRW;			/* Dot matrix Display Data Read/Write */
	M   READREG;		/* Register Read */
	M   TEST;		/* IC Test Command */
} CMDREG;

const CMDREG CmdRegDef = {  /***** Cmd Address & Data Initial *****/
		/*  size  cmd    dat1   dat2	 dat3	dat4  */
		/* SOFTRES		  */{1,  0x01,  0x00,  0x00,  0x00,	0x00},
		/* DDISPONOFF	  */{2,  0x02,  0x00,  0x00,  0x00,	0x00},
		/* DispDirection  */{2,  0x09,  0x00,  0x00,  0x00,	0x00},
		/* Reserved1	  */{4,  0x10,  0x1A,  0x00,  0x00,	0x00},
		/* Reserved2	  */{4,  0x12,  0x56,  0x00,  0x00,	0x00},
		/* Reserved3 	  */{2,  0x13,  0x00,  0x00,  0x00,	0x00},
		/* DSTBYONOFF	  */{2,  0x14,  0x01,  0x00,  0x00,	0x00},
		/* Reserved4	  */{2,  0x16,  0x00,  0x00,  0x00,	0x00},
		/* Reserved5	  */{2,  0x17,  0x00,  0x00,  0x00,	0x00},
		/* Reserved6	  */{2,  0x18,  0x0F,  0x00,  0x00,	0x00},
		/* Reserved7	  */{2,  0x1A,  0x04,  0x00,  0x00,	0x00},
		/* Reserved8	  */{2,  0x1C,  0x00,  0x00,  0x00,	0x00},
		/* WriteDirection */{2,  0x1D,  0x0B,  0x00,  0x00,	0x00},
		/* DispSizeX	  */{5,  0x30,  0x00,  0x10,  0x01,	0x0F},
		/* DispSizeY	  */{3,  0x32,  0x00,  0x3F,  0x00,	0x00},
		/* XBoxAdrsSTART  */{2,  0x34,  0x02,  0x00,  0x00,	0x00},
		/* XBoxAdrsEND	  */{2,  0x35,  0x21,  0x00,  0x00,	0x00},
		/* YBoxAdrsSTART  */{2,  0x36,  0x00,  0x00,  0x00,	0x00},
		/* YBoxAdrsEND	  */{2,  0x37,  0x3F,  0x00,  0x00,	0x00},
		/* XDispStart	  */{3,  0x38,  0x00,  0x00,  0x00,	0x00},
		/* YDispStart	  */{2,  0x39,  0x00,  0x00,  0x00,	0x00},
		/* Reserved9	  */{2,  0x48,  0x00,  0x00,  0x00,	0x00},
		/* SCLK 		  */{2,  0xD0,  0x80,  0x00,  0x00,	0x00},
		/* DDEN 		  */{2,  0xD2,  0x00,  0x00,  0x00,	0x00},
		/* Reserved10	  */{2,  0xD9,  0x00,  0x00,  0x00,	0x00},
		/* FDIM 		  */{2,  0xDB,  0x00,  0x00,  0x00,	0x00},
		/* Reserved11	  */{2,  0xDD,  0x8B,  0x00,  0x00,	0x00},
		/* DispMode 	  */{2,  0xE0,  0x00,  0x00,  0x00,	0x00},
		/* Reserved12	  */{2,  0xE2,  0x01,  0x00,  0x00,	0x00},
		/* Reserved13	  */{2,  0xE3,  0x00,  0x00,  0x00,	0x00},
		/* CAreaSizeY	  */{3,  0xE4,  0x3F,  0x3F,  0x00,	0x00},
		/* ColorChange1   */{2,  0xE5,  0x07,  0x00,  0x00,	0x00},
		/* ColorChange2   */{2,  0xE6,  0x07,  0x00,  0x00,	0x00},
		/* ColorChange3   */{2,  0xE7,  0x07,  0x00,  0x00,	0x00},
		/* Reserved14	  */{2,  0xFD,  0x10,  0x00,  0x00,	0x00},
		/* CmdEND		  */{0,  0x00,  0x00,  0x00,  0x00,	0x00},
		/* DataRW		  */{1,  0x08,  0x00,  0x00,  0x00,	0x00},
		/* READREG		  */{1,  0x20,  0x00,  0x00,  0x00,	0x00},
		/* TEST 		  */{2,  0xF0,  0x00,  0x00,  0x00,	0x00}
};

static bool g_power = true;

#define GPIO_DATA_OUT 0xd0018140
#define GPIO_BASE 0xd0018141
#define GPIO_SHIFT 1 /*GPIO_DATA_OUT-GPIO_BASE*/
#define GPIO_ARIA_SIZE 2

enum PIN{
	OLED_XRES = 48-32,
	OLED_XCS  = 49-32,
	OLED_A0   = 50-32,
	OLED_SI   = 51-32,
	OLED_SCK  = 52-32
};

enum FUNC{
	COMMAND,
	DATA
};

/**************************************/
/**** Serial I/F Definition       *****/
/**************************************/
struct SIF_REGISTER {
	BYTE TDR;   /* Transmission data register */
	bool TENA;  /* Transmission operation permission/prohibition */
	bool TEND;  /* Transmission end/unend */
} Sif;

struct file *file;

/**************************************/
/**** Global Variable Definition  *****/
/**************************************/
static CMDREG CmdReg;  /* CmdReg Work */

struct resource *g_el_addr;

void write_gpio(const u16 current_value ,enum PIN pin,bool val){
	const int shift = pin - (GPIO_SHIFT * 8);
	const u16 mask  = 1 << shift;
	u16 value = current_value;//ioread16(g_el_addr);
	if(val){
		value = value | mask;
	}else{
		value = value & (~mask);
	}
	iowrite16(value,g_el_addr);
	//wmb();
	//printk( KERN_INFO "fb_server : %d %d %u\n",current_value,shift,mask);
}

void gpio_set_value(enum PIN pin,bool val){
	write_gpio(ioread16(g_el_addr),pin,val);
}

/****************************************************************************
 *  NAME : trans_serial
 *  FUNC : Transfer for serial
 *  Inp  : pData, com_byte
 *         pData    : Transfer Start Address
 *         DataSize : Transfer Byte Count
 *  Out  : Nothing
 ****************************************************************************/
void trans_serial( BYTE *pData, WORD DataSize )
{
#ifdef CONFIG_BUFFALO_PLATFORM
	/* For devices in which the SPI is muxed on the MPP with other interfaces*/
	unsigned char spiId = m_spiId;
	mvSpiCsAssert(spiId);
	MV_STATUS status = mvSpiWrite(spiId, pData, DataSize);
	mvSpiCsDeassert(spiId);
#else
	//GPIO
	WORD i,j;
	gpio_set_value(OLED_SCK,false);
	for (j=0; j<DataSize; j++) {
		for (i=0; i<8; i++) {
			gpio_set_value(OLED_SI,(*pData) & (1<<(7-i)));
			gpio_set_value(OLED_SCK,true);
			gpio_set_value(OLED_SCK,false);
		}
		pData ++;
	}
#endif
}

/****************************************************************************
 *  NAME : com_trans
 *  FUNC : CMD or DATA Transfer for serial
 *  Inp  : dxc, pData, com_byte
 *         dxc       : 0:Cmd / 1:Parameter or Data
 *         pData     : Transfer Start Address
 *         DataSize  : Transfer Byte Count
 *  Out  : Nothing
 ****************************************************************************/
void com_trans( enum FUNC dxc, BYTE *pData, WORD DataSize )
{
#ifndef CONFIG_BUFFALO_PLATFORM
	gpio_set_value(OLED_XCS,false);          /* Set XCS : LOW */
#endif
	if (dxc == COMMAND) {     /* Command or Dot Data? */

		/******* Command Transfer for 8Bit Serial *******/
		gpio_set_value(OLED_A0, false);      /* Set A0  : LOW */
		trans_serial(pData, 1);

		if (DataSize >= 2) {
			gpio_set_value(OLED_A0, true);  /* Set A0  : HIGH */
			trans_serial(pData+1, DataSize-1);
		}

	} else {            /* Dot Data     */

		/******* Data Transfer for 8Bit Serial *******/
		gpio_set_value(OLED_A0, true);      /* Set A0  : HIGH */
		trans_serial(pData, DataSize);
	}
#ifndef CONFIG_BUFFALO_PLATFORM
	gpio_set_value(OLED_XCS, true);         /* Set XCS : HIGH */
#endif
}

/****************************************************************************
 *  NAME : EL_SifInit
 *  FUNC : Serial I/F Port Initialize
 *  Inp  : Nothing
 *  Out  : Nothing
 ****************************************************************************/
void EL_SifInit(void)           /* Serial I/F Port Initialize       */
{
	/*This software requires the achievement of the serial transfer of IC command & data by using
			the serial communications function of MPU. 
			Please follow the specification of MPU used about the initialization of the serial port. 
			The main setting of the serial transfer is as follows. 

			Communication mode  : Synchronous mode. 
			Data transfer       : MSB (bit 7) first. 
			Character length    : 8 bits.
			The bit rate        : Our sample is set to 8.3MHz.

			- Register & flag used
			�Transmission data register : Sif.TDR;
			�Transmission operation permission/prohibition  : Sif.TENA;
			�Transmission end/unend : Sif.TEND;*/
}

/****************************************************************************
 *  NAME : EL_RegInit
 *  FUNC : Set Default Value
 *  Inp  : Nothing
 *  Out  : Nothing
 ****************************************************************************/
void EL_RegInit(void)
{
	M	*reg;

	CmdReg = CmdRegDef;     /* Initial Value -> Work Copy */

	/***** Register Initialize *****/
	for (reg=&CmdReg.SOFTRES; reg!=&CmdReg.CmdEND; reg++){
		com_trans(COMMAND, &reg->cmd, reg->cmdsize);
	}
}

/*************************************************************************
  NAME : EL_DispSet
  FUNC : Display Parameter & Data  Set
  In   : UB Data(Write Data)
  Out  : Nothing
 *************************************************************************/
void EL_DispSet(BYTE *dat_buff)
{
	//色反転をデフォルトとする
	WORD i;
	static BYTE out_buff[EL_BYTE_SIZE];
	for (i=0; i<EL_BYTE_SIZE; i++){
		out_buff[i] = (~dat_buff[i]);
	}

	/***** Dot Matrix Display Data Write *****/
	com_trans(COMMAND, &CmdReg.DataRW.cmd, CmdReg.DataRW.cmdsize);
	com_trans(DATA, out_buff, EL_BYTE_SIZE);
}

/*************************************************************************
  NAME : EL_DispSetPattrn
  FUNC : Display Parameter & Data  Set
  In   : UB pattrn(Write Data Pattern)
  Out  : Nothing
 *************************************************************************/
void EL_DispSetPattrn(BYTE pattrn)
{
	WORD i;
	static BYTE dat_buff[EL_BYTE_SIZE];
	for (i=0; i<EL_BYTE_SIZE; i++){
		dat_buff[i] = pattrn;
	}
	EL_DispSet(dat_buff);
}


/****************************************************************************
  NAME : EL_DispStandby
  FUNC : set display control register   ( Cmd Reg Adr : DSTBYONOFF )
  In   : BOOL on_off - TRUE : Standby ON, FALSE : Standby OFF
  Out  : Nothing
 ****************************************************************************/
void EL_DispStandby(bool on_off)
{
	if (on_off)
		CmdReg.DSTBYONOFF.dat1 = 1;
	else
		CmdReg.DSTBYONOFF.dat1 = 0;

	com_trans(COMMAND, &CmdReg.DSTBYONOFF.cmd, CmdReg.DSTBYONOFF.cmdsize);

}

/****************************************************************************
  NAME : EL_DispControl
  FUNC : set display control register   ( Cmd Reg Adr : DDSPON/OFF )
  In   : BOOL on_off - TRUE : Display ON, FALSE : Display OFF
  Out  : Nothing
 ****************************************************************************/
void EL_DispControl(bool on_off)
{
	if (on_off)
		CmdReg.DDISPONOFF.dat1 = 1;
	else
		CmdReg.DDISPONOFF.dat1 = 0;

	com_trans(COMMAND, &CmdReg.DDISPONOFF.cmd, CmdReg.DDISPONOFF.cmdsize);
}

void EL_DispDimmer(BYTE bright){
	CmdReg.FDIM.dat1 = bright;
	com_trans(COMMAND, &CmdReg.FDIM.cmd, CmdReg.FDIM.cmdsize);
}

bool set_brightness(int bright){
	lock();
	if(g_power){
		if(0 <= bright && bright <= 0x0F){
			//EL_DispControl(false);  /* Display Off                      */
			//mdelay(10);
			//EL_DispStandby(true);	/* IC Standby On                    */

			CmdReg.FDIM.dat1 = bright;
			com_trans(COMMAND, &CmdReg.FDIM.cmd, CmdReg.FDIM.cmdsize);

			//EL_DispStandby(false);	/* IC Standby OFF                   */
			//EL_DispControl(true);   /* Display On                       */
			unlock();
			return true;
		}
	}
	unlock();
	return false;
}


bool set_power(bool p){
	lock();
	if(p){
		if(!g_power){
			EL_DispStandby(false);	/* IC Standby OFF                   */
			EL_DispControl(true);   /* Display On                       */
			g_power = true;
			unlock();
			return true;
		}
	}else{
		if(g_power){
			EL_DispControl(false);  /* Display Off                      */
			mdelay(10);
			EL_DispStandby(true);	/* IC Standby On                    */
			g_power = false;
			unlock();
			return true;
		}
	}
	unlock();
	return false;
}

int get_brightness(){
	return CmdReg.FDIM.dat1;
}

bool is_power(){
	return g_power;
}


static int dimmer_thread(void * v){
	BYTE bright=0;
	for(bright=0;bright<=0x0F;bright++){
		EL_DispDimmer(bright);
		msleep(75);
	}
	return 0;
}
struct task_struct *dimmer;

int on_display_init(){
	tz_init_lock();

	if(!request_mem_region(GPIO_BASE, GPIO_ARIA_SIZE, "el")) {
			//return -1;
	}
	g_el_addr = ioremap_nocache(GPIO_BASE, GPIO_ARIA_SIZE);//ノンキャッシュ指定でリマップした論理アドレス

	gpio_set_value(OLED_XRES,false);      /* Set RES  : LOW  */
#ifdef CONFIG_BUFFALO_PLATFORM
	//MV_STATUS status = mvSpiBaudRateSet(m_spiId,_8M);
	MV_STATUS status = mvSpiBaudRateSet(m_spiId,(20 << 20));
#else
	gpio_set_value(OLED_XCS, true);       /* Set RES  : HIGH */
#endif
	udelay(1);
	gpio_set_value(OLED_XRES, true);      /* Set RES  : HIGH */
	udelay(1100);

	EL_SifInit();           /* Serial I/F Port Initialize       */
	EL_RegInit();           /* IC Register Initialize           */

	EL_DispStandby(false);	/* IC Standby OFF                   */

	//EL_DispSetPattrn(0xFF); /* Display Parameter & Data  Set    */
	BYTE logo[] = {
		#include "logo.array"
	};
	EL_DispSet(logo);

	dimmer = kthread_run(dimmer_thread, NULL, "el_dimmer");

	EL_DispControl(true);   /* Display On                       */
	return 0;
}

int on_display_invalidate(struct fb_info *filp){
	lock();
	if(g_power){
		char* frame_buffer = filp->screen_base;
		unsigned int frame_size = filp->fix.smem_len;
		EL_DispSet(frame_buffer);
	}
	unlock();
	return 0;
}
int on_display_cleanup(){
	if(dimmer){
		kthread_stop(dimmer);
	}

#ifndef CONFIG_BUFFALO_PLATFORM
	gpio_set_value(OLED_XCS ,true);     /* Set XCS  : HIGH */
#endif
	gpio_set_value(OLED_XRES,false);    /* Set RES  : LOW */
	udelay(1);
#ifndef CONFIG_BUFFALO_PLATFORM
	gpio_set_value(OLED_XCS ,false);    /* Set LOW  : LOW */
#endif

	iounmap(g_el_addr);
	release_mem_region(GPIO_BASE, GPIO_ARIA_SIZE);

	filp_close(file,NULL);

	return 0;
}

#endif //DUMMY_DISP
