#include "fb_server.h"

struct fb_info *g_info;
const static char forword_bmp[38] = {0x42,0x4d,0x3e,0x8,0x0,0x0,0x0,0x0,0x0,0x0,0x3e,0x0,0x0,0x0,0x28,0x0,0x0,0x0,0x0,0x1,0x0,0x0,0x40,0x0,0x0,0x0,0x1,0x0,0x1,0x0,0x0,0x0,0x0,0x0,0x0,0x8,0x0,0x0};
const static char backword_bmp[38] = {0x42,0x4d,0x3e,0x8,0x0,0x0,0x0,0x0,0x0,0x0,0x3e,0x0,0x0,0x0,0x28,0x0,0x0,0x0,0x0,0x1,0x0,0x0,0xc0,0xff,0xff,0xff,0x1,0x0,0x1,0x0,0x0,0x0,0x0,0x0,0x0,0x8,0x0,0x0};

/*
 * ディスプレイ変動値
 */
static struct fb_var_screeninfo _var = {
		.xres = X_RES, /* visible resolution		*/
		.yres = Y_RES,
		.xres_virtual = X_RES, /* virtual resolution		*/
		.yres_virtual = Y_RES,
		.xoffset = 0, /* offset from virtual to visible */
		.yoffset = 0, /* resolution			*/

		.bits_per_pixel = BPS, /* guess what			*/
		.grayscale = 0, /* != 0 Graylevels instead of colors */

		.red = { .offset = COLOR_OFFSET * MONOBPS * 2, /* beginning of bitfield	*/
				.length = MONOBPS, /* length of bitfield		*/
				.msb_right = 0, /* != 0 : Most significant bit is */
		},/* bitfield in fb mem if true color, */
		.green = { .offset = COLOR_OFFSET * MONOBPS, /* beginning of bitfield	*/
				.length = MONOBPS, /* length of bitfield		*/
				.msb_right = 0, /* != 0 : Most significant bit is */
		}, /* else only length is significant */
		.blue = { .offset = 0, /* beginning of bitfield	*/
				.length = MONOBPS, /* length of bitfield		*/
				.msb_right = 0, /* != 0 : Most significant bit is */
		},
		//struct fb_bitfield transp;	/* transparency			*/

		//__u32 nonstd;					/* != 0 Non standard pixel format */

		.activate = FB_ACTIVATE_NOW, /* see FB_ACTIVATE_*		*/

		.height = -1, /* height of picture in mm    */
		.width = -1, /* width of picture in mm     */

		.accel_flags = FB_ACCEL_NONE, /* (OBSOLETE) see fb_info.flags */

		///* Timing: All values in pixclocks, except pixclock (of course) */
		//__u32 pixclock;				/* pixel clock in ps (pico seconds) */
		//__u32 left_margin;			/* time from sync to picture	*/
		//__u32 right_margin;			/* time from picture to sync	*/
		//__u32 upper_margin;			/* time from sync to picture	*/
		//__u32 lower_margin;
		//__u32 hsync_len;				/* length of horizontal sync	*/
		//__u32 vsync_len;				/* length of vertical sync	*/
		//.sync  = FB_SYNC_EXT,			/* see FB_SYNC_*		*/
		//.vmode = FB_VMODE_NONINTERLACED,/* see FB_VMODE_*		*/
		//__u32 rotate;					/* angle we rotate counter clockwise */
		//__u32 reserved[5];			/* Reserved for future compatibility */
};

/*
 * ディスプレイ固定値
 */
static struct fb_fix_screeninfo _fix = {
		.id = "EL Monitor", /* identification string eg "TT Builtin" */
		//unsigned long smem_start;		/* Start of frame buffer mem */
		/* (physical address) */
		//__u32 smem_len;				/* Length of frame buffer mem */
		.type = FB_TYPE_PACKED_PIXELS, /* see FB_TYPE_*		*/
		//__u32 type_aux;				/* Interleave for interleaved Planes */
		.visual = VISUAL, /* see FB_VISUAL_*		*/
		.xpanstep = 0, /* zero if no hardware panning  */
		.ypanstep = 0, /* zero if no hardware panning  */
		.ywrapstep = 0, /* zero if no hardware ywrap    */
		.line_length = X_LENGTH * BPS / 8, /* length of a line in bytes    */
		//unsigned long mmio_start;		/* Start of Memory Mapped I/O   */
		/* (physical address) */
		//__u32 mmio_len;				/* Length of Memory Mapped I/O  */
		//__u32 accel;					/* Indicate to driver which	*/
		/*  specific chip/card we have	*/
		//__u16 reserved[3];			/* Reserved for future compatibility */
};

/*!
* @breaf デバイスをflashした時に呼ばれる。このタイミングで描画を行う
* @param[put] filp ファイルポインタ
* @return エラーコード
*/
static int _sync(struct fb_info *filp) {
#ifdef DEBUG
	printk( KERN_INFO "%s : _fsync\n" ,DEV_MSG);
#endif
	//ディスプレイ更新を要求
	on_display_invalidate(filp);
	return 0;
}

/*!
* @breaf デバイスから読みこんだ時に呼ばれる
* @param[in] filp ファイルポインタ
* @param[out] buf 転送先ポインタ
* @param[in] filp 転送先ポインタのサイズ
* @param[out] pos 転送オフセット
* @return エラーコード・転送サイズ
*/
static ssize_t _read(struct fb_info* filp, char* buf, size_t count, loff_t* pos) {
	char* fb_vadderss = filp->screen_base;
	size_t fb_length = filp->fix.smem_len;
#ifdef DEBUG
	printk( KERN_INFO "%s : read",DEV_MSG);
#endif
	size_t copy_len = 0;
	if (fb_length <= *pos) {
		return 0;
	}
	int result = -EFAULT;
	copy_len = min(count,fb_length);
	if (copy_to_user(buf, &fb_vadderss[*pos], copy_len)) {
		printk( KERN_ERR "%s : copy_to_user failed\n" ,DEV_MSG);
		result = -EFAULT;
		goto end;
	}
	*pos += copy_len;
	result = copy_len;
end:
	return result;
}

/*!
* @breaf デバイスに書きこんだ時に呼ばれる
* @param[in] filp ファイルポインタ
* @param[in] buf  転送元ポインタ
* @param[in] filp 転送元ポインタのサイズ
* @param[out] pos 転送オフセット
* @return エラーコード・転送サイズ
*/
static ssize_t _write(struct fb_info* filp, const char* buf, size_t count, loff_t* pos) {
	unsigned long fb_start = filp->fix.smem_start;
	size_t fb_length = filp->fix.smem_len;
	size_t line_length = filp->fix.line_length;
#ifdef DEBUG
	printk( KERN_INFO "%s : write\n" ,DEV_MSG);
#endif
	size_t copy_len = 0;
	int result = -EFAULT;
	if (fb_length <= *pos) {
		//画面更新を要求
		_sync(filp);
		//正常完了
		result = 0;
		goto end;
	}

	//!TODO フレームバッファ＜転送バッファサイズと仮定してしまっている
	bool reverse = false;
	copy_len = count;
	size_t buf_len = min(count,fb_length);
	if(*pos==0 && count==2110){
		int h=62;
		if(memcmp(forword_bmp,buf,sizeof(forword_bmp))==0){
			buf = &buf[h];
			buf_len = min(count-h,fb_length);
			reverse = true;
		}if(memcmp(backword_bmp,buf,sizeof(backword_bmp))==0){
			buf = &buf[h];
			buf_len = min(count-h,fb_length);
		}
	}
	int y = 0;
	for(y=0;y<Y_RES;y++){
		int fy = reverse ? Y_RES-y-1 : y;
		if(buf_len<((y+1)*line_length)){
			continue;
		}
		if (copy_from_user(fb_start+(fy*line_length), buf+(y*line_length), line_length)) {
			printk( KERN_ERR "%s copy_from_user failed\n" ,DEV_MSG);
			result = -EFAULT;
			goto end;
		}
	}
	*pos += copy_len;

	//この条件は要検証
	if(fb_length <= count){
		//画面更新を要求
		_sync(filp);
	}
#ifdef DEBUG
	printk( KERN_INFO "%s : %d %d",DEV_MSG,fb_length,count);
#endif
	result = copy_len;
end:
	return result;
}

static void _vma_open(struct vm_area_struct *vm_area) {
#ifdef DEBUG
	printk( KERN_INFO "%s : vma_open\n" ,DEV_MSG);
#endif
}

static void _vma_close(struct vm_area_struct *vm_area) {
#ifdef DEBUG
	printk( KERN_INFO "%s : vma_close\n" ,DEV_MSG);
#endif
}

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,26))
static struct page _vma_nopage(struct vm_area_struct *vm_area, unsigned long address, int *type) {
#ifdef DEBUG
	printk( KERN_INFO "%s : vma_nopage\n" ,DEV_MSG);
#endif
	struct page *pageptr;
	unsigned long offset = address - vm_area->vm_start + VMA_OFFSET(vm_area);
	struct fb_info *filp = vm_area->vm_private_data;
	char* frame_buffer = filp->screen_base;
	pageptr = vmalloc_to_page(frame_buffer + offset);
	vm_area->page = pageptr;
	get_page(pageptr);
	return pageptr;
}
#else
static int _vma_fault(struct vm_area_struct *vm_area, struct vm_fault *vmf) {
#ifdef DEBUG
	printk( KERN_INFO "%s : vma_fault\n" ,DEV_MSG);
#endif
	unsigned long offset = vmf->pgoff << PAGE_SHIFT;
	struct fb_info *filp = vm_area->vm_private_data;
	char* fb_vadderss = filp->screen_base;
	struct page *pageptr = vmalloc_to_page(fb_vadderss + offset);
	vmf->page = pageptr;
	get_page(pageptr);
	return 0;
}
#endif

static struct vm_operations_struct remap_vm_ops = {
		.open =  _vma_open,
		.close = _vma_close,
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,26))
		.nopage= _vma_nopage,
#else
		.fault = _vma_fault,
#endif
};

/*!
* @breaf デバイスをメモリにマップ
* @param[in] vm_area メモリ管理構造体
* @return エラーコード
*/
static int _mmap(struct fb_info *filp, struct vm_area_struct *vm_area) {
	unsigned long fb_start = filp->fix.smem_start;
	size_t fb_length = filp->fix.smem_len;
#ifdef DEBUG
	printk( KERN_INFO "%s : mmap\n", DEV_MSG);
#endif
	if (!fb_start) {
		printk(KERN_ERR "%s : Memory not mapped!\n", DEV_MSG);
		return -EAGAIN;
	}
#ifdef DEBUG
	printk(KERN_INFO "%s : vm_start = %lu, vm_end = %lu\n", DEV_MSG, vm_area->vm_start, vm_area->vm_end);
#endif
	size_t vm_size = vm_area->vm_end - vm_area->vm_start;
	if (vm_size != max(fb_length,(size_t)PAGE_SIZE)) {
		printk(KERN_ERR "%s : sizes don't match (buffer size = %u, requested size = %u)\n", DEV_MSG, fb_length, vm_size);
		return -EAGAIN;
	}
	vm_area->vm_ops = &remap_vm_ops;
	vm_area->vm_flags |= VM_RESERVED;
	vm_area->vm_private_data = filp;
	_vma_open(vm_area);
	return 0;
}

/*!
* @breaf デバイスを制御する
* @param[in] filp ファイルポインタ
* @param[in] cmd  コントロールコマンド
* @param[out] p 転送元ポインタ
* @return エラーコード
*/
static long _ioctl(struct fb_info *filp, unsigned int cmd, unsigned long p) {
#ifdef DEBUG
	printk( KERN_INFO "%s : ioctl\n" ,DEV_MSG);
#endif
	int c2u(const void *from,const size_t len){
		size_t l = len;
		copy_to_user(p, from, l);
		return 0;
	}
	switch (cmd) {
	case FBIOGET_VSCREENINFO:
		return c2u(&filp->var, sizeof(filp->var));
	case FBIOGET_FSCREENINFO:
		return c2u(&filp->fix, sizeof(filp->fix));
	}
	return -EINVAL;
}

/*!
* @breaf デバイスのオープン時に呼ばれる
* @param[in] inode
* @param[in] filp ファイルポインタ
* @return 常に0
*/
static int _open(struct inode* inode, struct fb_info* filp) {
#ifdef DEBUG
	printk( KERN_INFO "%s : open\n" ,DEV_MSG);
#endif
	return 0;
}

/*!
* @breaf デバイスのクローズ時に呼ばれる
* @param[in] inode
* @param[in] filp ファイルポインタ
* @return 常に0
*/
static int _release(struct inode* inode, struct fb_info* filp) {
#ifdef DEBUG
	printk( KERN_INFO "%s : release\n",DEV_MSG);
#endif
	return 0;
}

static struct fb_ops _fb_ops = {
	.owner      = THIS_MODULE,
	.fb_read    = _read,
	.fb_write   = _write,
	.fb_mmap    = _mmap,
	.fb_ioctl   = _ioctl,
	.fb_open    = _open,
	.fb_release = _release,
};

/*!
* @breaf 明るさ制御用のprocへの書き込み時に呼ばれる
* @param[in] filp   ファイルポインタ
* @param[in] buffer 転送元ポインタ
* @param[in] count  転送元サイズ
* @return エラーコード・転送サイズ
*/
static int brightness_write_proc(struct file *filp, const char *buffer, unsigned long count, void *data)
{
	char buf[2];
	strncpy(buf,buffer,1);/*count==1*/
	int brightness = simple_strtol( buf, NULL, 16 );
	if(set_brightness(brightness)){
		printk( KERN_INFO "%s : brightness %x\n" ,DEV_MSG,brightness);
	}
	return count;
}

/*!
* @breaf 明るさ制御用のprocからの読み出し時に呼ばれる
* @param[out] buffer 転送先ポインタ
* @param[out] start  転送位置
* @param[in] off     転送オフセット
* @param[in] count   転送先サイズ
* @param[out] eof    完了マーク
* @return エラーコード・転送サイズ
*/
static int brightness_read_proc(char *buffer, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;
	len += sprintf(buffer + len, "%x\n",get_brightness());
	if (off + count >= len){
		*eof = 1;
	}
	if (len < off){
		return 0;
	}
	*start = buffer + off;
	return ((count < len - off) ? count : len - off);
}

/*!
* @breaf 電源制御用のprocへの書き込み時に呼ばれる
* @param[in] filp   ファイルポインタ
* @param[in] buffer 転送元ポインタ
* @param[in] count  転送元サイズ
* @return エラーコード・転送サイズ
*/
static int power_write_proc(struct file *filp, const char *buffer, unsigned long count, void *data)
{
	char buf[2];
	strncpy(buf,buffer,1);/*count==1*/
	int power = simple_strtol( buf, NULL, 10 );
	if(set_power(power)){
		printk( KERN_INFO "%s : power %x\n" ,DEV_MSG,power);
	}
	return count;
}

/*!
* @breaf 電源制御用のprocからの読み出し時に呼ばれる
* @param[out] buffer 転送先ポインタ
* @param[out] start  転送位置
* @param[in] off     転送オフセット
* @param[in] count   転送先サイズ
* @param[out] eof    完了マーク
* @return エラーコード・転送サイズ
*/
static int power_read_proc(char *buffer, char **start, off_t off, int count, int *eof, void *data)
{
	int len = 0;
	len += sprintf(buffer + len, "%x\n",is_power());
	if (off + count >= len){
		*eof = 1;
	}
	if (len < off){
		return 0;
	}
	*start = buffer + off;
	return ((count < len - off) ? count : len - off);
}

static void _deferred_io(struct fb_info *info,struct list_head *pagelist){
#ifdef DEBUG
	printk( KERN_INFO "%s : _deferred_io\n" ,DEV_MSG);
#endif
	//手抜き
	_sync(info);
}

#ifdef CONFIG_FB_DEFERRED_IO
static struct fb_deferred_io _defio = {
	.delay       = HZ,
	.deferred_io = _deferred_io,
};
#endif

static int _probe(struct platform_device *pdev){
	//バーチャルメモリの確保（フレームバッファの実体）
	const size_t fb_length = PIXEL_BYTES;
	const unsigned long fb_start = vmalloc(max(fb_length,(size_t)PAGE_SIZE));
	if (!fb_start) {
		printk( KERN_INFO "%s : can't alloc mem.\n" ,DEV_MSG);
		return -EBUSY;
	}
	//メモリ初期化
	memset(fb_start,0xFF,fb_length);
#ifdef DEBUG
	printk( KERN_INFO "%s : vaddress = %lu\n",DEV_MSG, fb_start );
	printk( KERN_INFO "%s : length  = %u\n",  DEV_MSG, fb_length);
#endif
	int ret = -EINVAL;
	{
		struct device *dev = &pdev->dev;
		struct fb_info *info = framebuffer_alloc(0, dev);
		if (!info){
			ret = -ENOMEM;
			goto err;
		}
		_fix.smem_start = fb_start;
		_fix.smem_len = fb_length;
		info->var = _var;
		info->fix = _fix;
		info->fbops = &_fb_ops;
		info->flags = FBINFO_DEFAULT;  /* not as module for now */
		info->screen_base = (char *) info->fix.smem_start;
#ifdef CONFIG_FB_DEFERRED_IO
		info->fbdefio = &_defio;
#endif
#ifdef CONFIG_FB_DEFERRED_IO
		fb_deferred_io_init(info);
#endif
		if (register_framebuffer(info) < 0) {
			printk(KERN_ERR "%s : Unable to register frame buffer\n" ,DEV_MSG);
			framebuffer_release(info);
			ret = -EINVAL;
			goto err;
		}
		printk( KERN_INFO "%s : fb%d : frame buffer alive and kicking !\n" ,DEV_MSG,info->node);
#ifdef DEBUG
		printk( KERN_INFO "%s : address = %lu\n", DEV_MSG, info->fix.smem_start );
#endif
		g_info = info;
	}
	{
		//////////////////////////
		//procディレクトリの準備
		//初回起動時はauto_create_proc_entryがうまくいかない
		struct proc_dir_entry *ent = proc_mkdir(PROC_DIR_PATH, 0);
		if (!ent){
			printk( KERN_ERR "%s : cannot create proc dir error\n" ,DEV_MSG);
			ret = -EBUSY;
			goto err;
		}
	}
	{
		//////////////////////////
		//明るさ調整
		//struct proc_dir_entry *ent = auto_create_proc_entry(PROC_BRIGHTNESS_PATH);
		struct proc_dir_entry *ent = create_proc_entry(PROC_BRIGHTNESS_PATH,0,0);
		if (!ent){
			printk( KERN_ERR "%s : cannot create proc error\n" ,DEV_MSG);
			ret = -EBUSY;
			goto err;
		}
		ent->read_proc = brightness_read_proc;
		ent->write_proc= brightness_write_proc;
	}
	{
		//////////////////////////
		//電源調整
		//struct proc_dir_entry *ent = auto_create_proc_entry(PROC_POWER_PATH);
		struct proc_dir_entry *ent = create_proc_entry(PROC_POWER_PATH,0,0);
		if (!ent){
			printk( KERN_ERR "%s : cannot create proc error\n" ,DEV_MSG);
			ret = -EBUSY;
			goto err;
		}
		ent->read_proc = power_read_proc;
		ent->write_proc= power_write_proc;
	}
	on_display_init();
	return 0;
err:
	//バーチャルメモリ解放
	if(fb_start){
		vfree(fb_start);
	}
	return ret;
}

static struct platform_driver _driver = {
	.probe  = _probe,
	.driver = {
		.name = DEV_NAME,
	},
};

static struct platform_device _device = {
	.name = DEV_NAME,
};

////////////////////////////////////////
//module

/*!
* @breaf デバイス開放時に呼ばれる
* @return エラーコード
*/
static void _exit(void) {
	on_display_cleanup();

	remove_proc_entry(PROC_BRIGHTNESS_PATH, 0);
	remove_proc_entry(PROC_POWER_PATH, 0);

	struct fb_info *info = g_info;
	if(info){
		//バーチャルメモリ解放
		if(info->fix.smem_start){
			vfree(info->fix.smem_start);
		}
#ifdef CONFIG_FB_DEFERRED_IO
		fb_deferred_io_cleanup(info);
#endif
		framebuffer_release(info);
		g_info = NULL;
	}

#ifdef DEBUG
	printk( KERN_INFO "%s : cleanup\n" ,DEV_MSG);
#endif
}

/*!
* @breaf デバイス初期化時に呼ばれる
* @return エラーコード
*/
static int _init(void) {
	//printk( KERN_ERR "%s : init\n" ,DEV_MSG);
	int ret = -EINVAL;
	//フレームバッファとしての初期化
	if (fb_get_options(DEV_NAME, NULL)){
		ret = -ENODEV;
		goto err;
	}
	ret = platform_driver_register(&_driver);
	if (ret) {
		goto err;
	}
	ret = platform_device_register(&_device);
	if (ret){
		platform_driver_unregister(&_driver);
		goto err;
	}
	printk("%s %s Ver.%s installed.\n", DRIVER_NAME, AUTHOR, BUFFALO_DRIVER_VER);
	return 0;
err:
	_exit();//ロールバック
	return ret;
}

module_init(_init);
module_exit(_exit);

MODULE_AUTHOR(AUTHOR);
MODULE_DESCRIPTION(DRIVER_NAME);
MODULE_LICENSE("Proprietary");
MODULE_VERSION(BUFFALO_DRIVER_VER);
