#include "audio_nas_keycode.h"
#include "input_device_server.h"

/*
 * audio_nas_event.c
 * Created on: 2013/04/10
 * Author: Kenji TAKAHASHI
 * 機種依存の設定は全てここに集約される
 */

/*0xd0018100  gpio data out
0xd0018140  gpio data out
0xd0018104  gpio data out en
0xd0018144  gpio data out en
0xd0018108  brink
0xd0018148  brink
0xd0018110  gpio data in
0xd0018150  gpio data in*/

#define GPIO_BASE 0xd0018110
#define GPIO_SHIFT 1
#define GPIO_ARIA_SIZE 2

enum PIN{
	GPIO_PANEL_UP     = 13,
	GPIO_PANEL_DOWN   = 14,
	GPIO_PANEL_RIGHT  = 15,
	GPIO_PANEL_LEFT   = 16,
	GPIO_PANEL_POWER  = 12,
};

int on_key_init(){
	//printk( KERN_ERR "read_pin : init");
	if(!request_mem_region(GPIO_BASE+GPIO_SHIFT, GPIO_ARIA_SIZE, "key")) {
		//return -1;
	}
	//g_el_addr = ioremap_nocache(GPIO_BASE, GPIO_ARIA_SIZE);//ノンキャッシュ指定でリマップした論理アドレス
	return 0;
}

int on_key_cleanup(){
	//printk( KERN_ERR "read_pin : cleanup");
	//iounmap(g_el_addr);
	release_mem_region(GPIO_BASE+GPIO_SHIFT, GPIO_ARIA_SIZE);
	return 0;
}

void write_pin(int pin,bool b){
	if(pin<32){
		struct resource *addr = ioremap_nocache(0xd0018100, 4);
		u32 mask = 1 << pin;

		u32 value = ioread32(addr);
		if(b){
			value = value | mask;
		}else{
			value = value & (~mask);
		}
		iowrite32(value, addr);
	}else if(pin<64){
		struct resource *addr = ioremap_nocache(0xd0018140, 4);
		u32 mask = 1 < (pin-32);

		u32 value = ioread32(addr);
		if(b){
			value = value | mask;
		}else{
			value = value & (~mask);
		}
		iowrite32(value, addr);
	}
}

static bool read_pin(enum PIN pin,u16 value){
	const int shift = pin - (GPIO_SHIFT*8);
	//printk( KERN_ERR "read_pin : shift %d v %d\n",shift,((value << shift) & 1));
	return ((value >> shift) & 1) == 0;
}

void read_pin_values(volatile union key_down_uni *key){
	struct resource *addr = ioremap_nocache(GPIO_BASE+GPIO_SHIFT, GPIO_ARIA_SIZE);
	//printk( KERN_ERR "read_pin : addr %u\n",addr);
	u16 value     = ioread16(addr);
	iounmap(addr);
	//printk( KERN_ERR "read_pin : value %x\n",value);
	key->list.key_up       = read_pin(GPIO_PANEL_UP   ,value);
	key->list.key_down     = read_pin(GPIO_PANEL_DOWN ,value);
	key->list.key_right    = read_pin(GPIO_PANEL_RIGHT,value);
	key->list.key_left     = read_pin(GPIO_PANEL_LEFT ,value);
	key->list.key_poweroff = !read_pin(GPIO_PANEL_POWER,value);//ここだけ反転
	//printk( KERN_ERR "read_pin_values : %d %d %d %d %d\n",now.key_up,now.key_down,now.key_right,now.key_left,now.key_poweroff);
}

